/*
 * Copyright (c) 2001-2004 Ant-Contrib project.  All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.antcontrib.property;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/**
 * Provides basic math functions. Simple calculations can be done via attributes
 * only, more complex formulas can be set up via nested Ops.
 * <p>Developed for use with Antelope, migrated to ant-contrib Oct 2003.
 * @author Dale Anson, danson@germane-software.com
 * @version $Revision: 1.4 $
 */
public class MathTask extends Task {

   // storage for result
   private String result = null;

   // storage for operation -- only one allowed
   private Op op = null;

   // storage for operation passed as attribute
   private String operation = null;

   // storage for an operand passed as an attribute
   private String operand1 = null;

   // storage for an operand passed as an attribute
   private String operand2 = null;

   // datatype for the result
   private String datatype = null;

   // should the StrictMath library be used?
   private boolean strict = false;

   /**
    * Sets the name of the property to store the result in. This is stored 
    * in a user property, so is reusable.
    * @param name the name of a property to set for a result.   
    */
   public void setResult( String name ) {
      result = name;
   }

   /**
    * Sets the datatype of this calculation. Allowed values are
    * "int", "long", "float", or "double". Optional, if
    * used, will be applied to all numbers in this math operation.
    */
   public void setDatatype( String type ) {
      if ( type.equals( "int" ) )
         datatype = "int";
      else if ( type.equals( "long" ) )
         datatype = "long";
      else if ( type.equals( "float" ) )
         datatype = "float";
      else if ( type.equals( "double" ) )
         datatype = "double";
      else
         throw new BuildException( "Invalid datatype: " + type +
               ". Must be one of int, long, float, or double." );
   }

   /**
    * Set an operand as an attribute. This is for convenience, if used,
    * it overrides any nested Ops. Must parse to a number. 
    */
   public void setOperand1( String op ) throws BuildException {
      operand1 = op;
   }

   /**
    * Set an operand as an attribute. This is for convenience, if used,
    * it overrides any nested Ops. Must parse to a number. 
    */
   public void setOperand2( String op ) throws BuildException {
      operand2 = op;
   }

   /**
    * Set an operation as an attribute. This is for convenience, if used, it
    * overrides any nested Ops. 
    * @param op any operation allowed by Op.   
    */
   public void setOperation( String op ) {
      operation = op;
   }

   /**
    * Add a nested operation. Only one operation is allowed at a time.
    * @param op the operation to add.
    */
   public void addConfiguredOp( Op op ) {
      if ( this.op != null )
         throw new BuildException( "Only one operation allowed at a time!" );
      if ( datatype != null )
         op.setDatatype( datatype );
      this.op = op;
   }

   /**
    * Use the StrictMath library.   
    */
   public void setStrict( boolean b ) {
      strict = b;
   }

   public void execute() {
      if ( result == null )
         throw new BuildException( "Property name for result is null." );
      if ( datatype == null )
         datatype = "double";
      if ( operation != null ) {
         // operation as attribute overrides nested Op
         op = new Op();
         if ( datatype == null )
            datatype = "double";
         op.setDatatype( datatype );
         op.setOp( operation );
         if ( operand1 != null ) {
            Num num = new Num();
            num.setValue( operand1 );
            op.addNum( num );
         }
         if ( operand2 != null ) {
            Num num = new Num();
            num.setValue( operand2 );
            op.addNum( num );
         }
      }
      if ( op == null )
         throw new BuildException( "Nothing to do!" );
      op.setDatatype( datatype );
      op.setStrict( strict );
      Num num = op.calculate();
      getProject().setUserProperty( result, num.getValue().toString() );
   }

}
